#
# =======================================================================
# PACKAGE:	fstat -- Print File Status
# SH-PROC:	Demo Script #4 (find files with identical contents)
# VERSION:	%R% (%E%)
# =======================================================================
# Copyright (1994): Martin Weitzel, Darmstadt, Germany
#
# This script contains various uses of the "fstat" command to find files
# that are NOT linked but nevertheless have identical contents.  File
# names are generated by "find", then all linked names are filtered out,
# then files are grouped according to size. 
#
# Bugs: The algorithm will break if there are blanks or tabs embedded
#	within a file name.

# The following function is needed in various places.  It groups its
# input by lines which an identical first field.  Input must be sorted.
#
group1() {
	read lgrp name && {
		while
			echo "$name"
			read grp name
		do
			test "$lgrp" != "$grp" && {
				echo
				lgrp="$grp"
			}
		done
		echo
	}
}

find $* -size +0 -print |	# traverse directory tree   
fstat |				# add dev/ino to name
sort -u +0 -1 |			# keep uniq dev/ino pairs only
cut -d' ' -f2 |			# keep file names only
fstat '+%s@%H %F' |		# add size and checksum to name
sort |		 		# sort identical entries in sequence
group1 |			# build groups by first field
awk 'BEGIN {			# (start-up)
	FS="\n";			# (fields are lines)
	RS="";				# (records come as groups of lines)
}
NF > 1 {			# if at least two names
	for (i = 1; i <= NF; ++i) {	# for all names
		if (!$i) continue;		# skip processed ones
		same = $i;			# start subgroup
		for (j = i+1; j <= NF; ++j) {	# for all unprocessed
						# check for same contents
			if (system("cmp -s '\''"$i"'\'' '\''"\
						$j"'\''") == 0) {
				same = same"\n"$j;	# store to subgroup
				$j = "";		# mark processed
			}
		}
		if (index(same, "\n"))		# if at least two names
			print same "\n";		# print subgroup
	}
}' |
awk 'BEGIN {			# (start-up)
	FS="\n";			# (fields are lines)
	RS="";				# (records come as groups of lines)
}
{
	min = $1;		# find smallest name
	for (i = 2; i <= NF; ++i)
		if (min > $i) min = $i;
				# print smallest name as leader
	for (i = 1; i <= NF; ++i)
		print min, $i;
}' |
sort |				# sort first by group, then by name
group1				# build groups by first field
